package app;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Logger;

import logic.sequence.SequenceBuilder;
import logic.sequence.impl.DNASequenceImpl;
import logic.tfbs.TFPatternBuilder;
import logic.tfbs.impl.PSSMImpl;
import logic.tfbs.impl.TFPatternBuilderImpl;
import logic.tfbs.impl.PSSMImpl.PSSMIOFactory;

public class Config {
	
	public static final String APP_NAME = "Billboard";
	public static final String VERSION = "1.0";

	private static String propertyFilePath = "res/app.properties";
	private static Properties props = null;
	
	public static TFPatternBuilder tfpb = new TFPatternBuilderImpl();
	public static SequenceBuilder seqb = DNASequenceImpl.getBuilder();
	public static PSSMIOFactory piof = new PSSMImpl.PSSMIOFactory();
	public static Logger l = Logger.getLogger(Config.APP_NAME);
	
	public static Properties loadAppProperties() throws FileNotFoundException, IOException {
		return loadAppPropertiesFromFile(propertyFilePath);
	}
	
	public static Properties loadAppPropertiesFromFile(String filePath) throws FileNotFoundException, IOException {
		props = new Properties();
		props.load(new FileInputStream(filePath));
		return props;
	}
	
	public static Properties getAppProperties() {
		return props;
	}
	
	public static String getProperty(String propertyName) {
		return props.getProperty(propertyName);
	}

	public static final class PropName {
		/**
		 * The string representation of a path to a file which
		 * indicates sequence pairs to be compared.
		 * File format is (EnsemblStableID1\tEnsemblStableID2\n)+.
		 */
		public static final String PAIRS_FILE = "pairsFile";
		public static final String DEFAULT_TF_DIR = "defaultTFDir";
		public static final String TF_SET = "tfSet";

		public static final String SEQUENCES_DIR = "sequencesDirectory";
		public static final String DOWNLOAD_SEQUENCES = "downloadSequences";
		public static final String BACKGROUND_SEQUENCES_DIR = "backgroundSequencesDirectory";
				
		public static final String LOG_DIR = "logDir";
		public static final String OUTPUT_DIR = "outputDir";
		
		public static final String PARAMETERS_FILE = "scoreParametersFile";

		public static final String UPSTREAM_LENGTH = "upstreamLength";
		public static final String DOWNSTREAM_LENGTH = "downstreamLength";

		public static final String THREADS = "threads";

		public static final String THRESHOLD_TYPE = "thresholdType";
		public static final String PSSM_THRESHOLD = "absoluteThresholdValue";

		public static final String PROPERTIES_FILE = "properties";
	}
}
