package app;

import java.io.File;
import java.io.FileFilter;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import logic.sequence.DNASequence;
import logic.sequence.impl.PreparedSequence;
import logic.sequence.io.FastaSequenceReader;
import logic.tfbs.PatternSet;
import logic.tfbs.impl.Threshold;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import app.exception.ConfigurationException;
import app.tools.KnuthShuffle;

public class LocalBackgroundSequenceManager implements
		BackgroundSequenceManager {
	private Log l = LogFactory.getLog(getClass());
	
	private Set<PreparedSequence> sequences = new HashSet<PreparedSequence>();

	public LocalBackgroundSequenceManager(String path, Threshold threshold, PatternSet patternSet) throws ConfigurationException {
		Map<String, DNASequence> sequences = new HashMap<String, DNASequence>();
		File dir = new File(path);
		File[] seqFiles = dir.listFiles(new FileFilter() {
			public boolean accept(File f) {
				return f.isFile() && !f.isHidden();
			}
		});
		if (seqFiles == null) {
			throw new ConfigurationException("Could not list files for " + path + ". Check if the specified path is correct.");
		}
		for (File file : seqFiles) {
			try {
				for (DNASequence s : FastaSequenceReader.readDNASeqFromFile(file, false)) {
					sequences.put(s.getName(), s);
				}
			} catch (Exception e) {
				l.error(e);
			}
		}
		if (sequences.size() < 100) {
			throw new ConfigurationException("Too few sequences read from " + path +
					" (should be 100 or more is " + sequences.size() + ").");
		} else {
			int[] shuffle = KnuthShuffle.generateShuffle(sequences.size());
			DNASequence[] values = sequences.values().toArray(new DNASequence[0]);
			for (int i = 0; i < 100; i++) {
				DNASequence sequence = values[shuffle[i]];
				PreparedSequence prepared = new PreparedSequence(sequence, patternSet, threshold);
				prepared.getPatterns(); // called for side-effects
				this.sequences.add(prepared);
			}
		}
	}

	public Set<PreparedSequence> getSequences() {
		return sequences;
	}

}
