package app;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import logic.sequence.DNASequence;
import logic.sequence.io.EnsemblDBReader;
import logic.tfbs.PatternSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import app.ensembl.IdNotRecognizedException;
import app.ensembl.Species;
import app.exception.BillboardException;
import app.exception.UserDataException;

public class Registry {
	
	@SuppressWarnings("unused")
	private static Log l = LogFactory.getLog(Registry.class);
	
	private Map<String, PatternSet> patternSets = new HashMap<String, PatternSet>();
	private Map<String, Map<String, DNASequence>> backgroundSeqs = new HashMap<String, Map<String,DNASequence>>();
	
	
	public void registerBgSequence(DNASequence s) throws DuplicateSequenceIdentifier, IdNotRecognizedException {
		Species species = Species.getForEnsembleId(s.getName());
		if (species != null) {
			Map<String, DNASequence> name = backgroundSeqs.get(species.getDataSet());
			if (name == null) {
				name = new HashMap<String, DNASequence>();
				backgroundSeqs.put(species.getDataSet(), name);
			}
			if (name.put(s.getName(), s) != null)
				throw new DuplicateSequenceIdentifier(s.getName());
			l.info("Background sequence " + s.getName() + " registered");
		}
	}
	
/*	public DNASequence[] getAllSequences() {
		return seqMap.values().toArray(new DNASequence[0]);
	}*/
	
	public DNASequence[] getBgSequences(Species s) {
		if (backgroundSeqs == null)
			return null;
		return backgroundSeqs.get(s.getDataSet()).values().toArray(new DNASequence[0]);
	}
	
	private Registry() {}
	
	private static Registry instance = null;
	public static Registry getRegistry() {
		if (instance == null) {
			instance = new Registry();
		}
		return instance;
	}
	
	public PatternSet getPatternSet(String setName) {
		return patternSets.get(setName);
	}
	
	public Set<String> getPatternSetNames() {
		return patternSets.keySet();
	}

	public void registerPatternSet(PatternSet set) throws UserDataException {
		if (patternSets.get(set.getName()) != null)
			throw new UserDataException("Duplicate pattern set names: " + set.getName());
		patternSets.put(set.getName(), set);
		l.info("Pattern set " + set.getName() + " registered");
	}
}
