package app.billboard;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

import logic.sequence.DNASequence;
import logic.sequence.io.EnsemblDBReader;
import logic.sequence.io.db.QueryException;
import logic.sequence.io.db.sql.entity.Gene;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.BasicConfigurator;

import app.ensembl.Species;
import app.exception.BillboardException;
import app.test.ArgumentParser;
import app.test.ArgumentParser.ArgumentParserException;
import app.tools.KnuthShuffle;

public class RandomSequenceDownloader {

	/**
	 * @param args
	 */
	
	private static final Log l = LogFactory.getLog(RandomSequenceDownloader.class);
	
	static {
		BasicConfigurator.configure();
	}
	
	public static void main(String[] args) throws ArgumentParserException {
//		try {
//			DNASequence d = EnsemblDBReader.getReader().getByStableId("ENSMUSG00000074792", 0, 10000, true);
//			d.exportFastaStream(System.out);
//		} catch (SQLException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		} catch (QueryException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		} catch (IOException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		} catch (BillboardException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
//		System.exit(1);
		String[] requiredParameters = new String[] { "species", "output", "before", "after", "amount" };
		ArgumentParser ap = new ArgumentParser(args, requiredParameters);
		String species = ap.getArgument("species");
		String outputDir = ap.getArgument("output");
		String beforeString = ap.getArgument("before");
		String afterString = ap.getArgument("after");
		String amountString = ap.getArgument("amount");
		Species s;
		try {
			s = Species.valueOf(species);
		} catch (IllegalArgumentException e) {
			l.error("No species for name " + species);
			l.error("Available options are:");
			for (Species sp : Species.values()) {
				l.error(sp);
			}
			return;
		}
		File output = new File(outputDir);
		if (!output.isDirectory()) {
			l.error(outputDir + " should be a directory.");
			return;
		}
		int before;
		int after;
		int amount;
		try {
			before = Integer.parseInt(beforeString);
			after = Integer.parseInt(afterString);
			amount = Integer.parseInt(amountString);
		} catch (NumberFormatException e) {
			l.error("'before', 'after' and 'amount' parameters should both be of type integer.", e);
			return;
		}
		EnsemblDBReader edbr;
		try {
			edbr = EnsemblDBReader.getReader();
		} catch (BillboardException e) {
			l.error("Could not instantiate ensembl database reader.", e);
			return;
		}
		List<Gene> genes;
		try {
			genes = edbr.getGeneIdentifiers(s);
		} catch (SQLException e) {
			l.error("Could not retrieve gene identifiers from ensembl database.", e);
			return;
		}
		l.info(genes.size() + " genes retrieved from ensembl database for species " + s);
		int[] shuffle = KnuthShuffle.generateShuffle(genes.size());
		File result = new File(output, "random.fasta");
		BufferedWriter bw;
		try {
			bw = new BufferedWriter(new FileWriter(result));
		} catch (IOException e) {
			l.error("Could not create output file " + result.getPath() + ".", e);
			return;
		}
		int count = 0;
		for (int i = 0; i < shuffle.length && count < amount; i++) {
			try {
				DNASequence sequence = edbr.getByStableId(genes.get(shuffle[i]).stableId, before, after, false);
				sequence.exportFastaStream(bw);
				count++;
			} catch (Exception e) {
				l.warn(genes.get(i).stableId + " rejected.", e);
			}
		}
		try {
			bw.flush();
			bw.close();
		} catch (IOException e) {
			l.warn("Error occured while closing output stream.", e);
		}
		if (count != 100) {
			l.error("Could not download " + amount + " sequences. Got " + count + ".");
		}
	}
}
