package app.billboard.homologs;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class CigarLine {
	private static final Log l = LogFactory.getLog(CigarLine.class);
	private String enc;
	private String inflated;
	private boolean strand;
	public CigarLine(String enc, boolean strand) {
		this.enc = enc;
		this.strand = strand;
		initialise();
	}
	
	public int actualLength() {
		return inflated.length();
	}
	
	public String getInflated() {
		return inflated;
	}
	
	private void initialise() {
		parse();
	}
	private void parse() {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < enc.length(); i++) {
			char c = enc.charAt(i);
			if (c == 'M' || c == 'D')
				sb.append(c);
			else if (Character.isDigit(c)) {
				StringBuilder sbdigit = new StringBuilder();
				int j;
				for (j = i; j < enc.length() && Character.isDigit(enc.charAt(j)); j++)
					sbdigit.append(enc.charAt(j));
				i = j;
				int times = Integer.parseInt(sbdigit.toString());
				for (int m = 0; m < times; m++)
					sb.append(enc.charAt(i));
			} else
				throw new Error();
		}
		inflated = sb.toString();
	}

	public int getOffset(int dist) {
		int hOff = 0, i = 0;
		for (; i < this.actualLength() && hOff < dist; i++) {
			if (inflated.charAt(i) == 'M')
				hOff++;
		}
		if (strand)
			return i;
		else
			return actualLength() - i - 1;
	}

	public int countOffset(int offset) {
		if (offset >= actualLength())
			throw new IllegalArgumentException();
		int result = 0;
		for (int i = 0; i < offset; i++) {
			if (inflated.charAt(i) == 'M')
				result++;
		}
		return result;
	}
	
}
