package app.graphics;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;

import javax.swing.JPanel;

import app.intf.Parameters;

public class HeatMapPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1052153362615852734L;
	private Image img;
	private int scoreX = -1, scoreY = -1;
	private double curScore = Double.NaN;
	private int jump;
	private double curSScore;

	public HeatMapPanel(Image img, Parameters params) {
		this.setLayout(null);
		Dimension dim = new Dimension(img.getWidth(), img.getHeight());
		this.setSize(dim);
		this.setPreferredSize(dim);
		this.setMinimumSize(dim);
		this.setMaximumSize(dim);
		this.img = img;
		this.setOpaque(false);
		this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		
		
		addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				super.mouseClicked(e);
				displayScore(e.getX(), e.getY());
			}
		});
		
		addMouseMotionListener(new MouseMotionListener() {

			public void mouseDragged(MouseEvent e) {
				displayScore(e.getX(), e.getY());
			}

			public void mouseMoved(MouseEvent e) {
				
			}
			
		});
		jump = params.getJump();
	}
	
	protected void displayScore(int x, int y) {
		this.scoreX = x;
		this.scoreY = y;
		this.curScore = img.getPixelValue(x, y);
		this.curSScore = img.getSPixelValue(x, y);
		repaint();
	}

	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.drawImage(img.createImage(), 0, 0, null);
		if (!(Double.isNaN(this.curScore))) {
			TextLayout tl = new TextLayout(this.curScore + " (" + this.curSScore + ") (" + scoreX/2 * jump + "," + scoreY/2 * jump + ")", g2.getFont(), g2.getFontRenderContext());
			Rectangle2D bounds = tl.getBounds();
			g2.setColor(Color.BLACK);
			tl.draw(g2, (float) Math.min(scoreX, this.getWidth() - bounds.getWidth()), (float) Math.max(bounds.getHeight(), scoreY));
//			g2.drawString(this.curScore + " (" + scoreX/2 * jump + "," + scoreY/2 * jump + ")", Math.min(this.scoreX, this.scoreY);
		}
	}
}
