package app.graphics;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.util.List;

import logic.alignment.AlignmentPrepared.Footprint;

public class Image {
	private double[][] m;
	private double max = Double.NEGATIVE_INFINITY;
	private int red = Integer.MAX_VALUE << 16;
	private BufferedImage bi = null;
	private int scale = 2;
	private double[][] sMatrix;

	public Image(double[][] matrix, double[][] sMatrix) {
		this.m = matrix;
		this.sMatrix = sMatrix;
		findMax();
		bi = createImage();
	}
	
	public double getPixelValue(int x, int y) {
		if (x >= 0 && y >= 0)
			if (x/2 < m.length)
				if (y/2 < m[x/2].length)
					return m[x/2][y/2];
		return Double.NaN;		
	}
	
	public double getSPixelValue(int x, int y) {
		if (x >= 0 && y >= 0)
			if (x/2 < sMatrix.length)
				if (y/2 < sMatrix[x/2].length)
					return sMatrix[x/2][y/2];
		return Double.NaN;
	}
	
	public int getHeight() {
		return bi.getHeight();
	}
	
	public int getWidth() {
		return bi.getWidth();
	}
	
	private void findMax() {
		for (int i = 0; i < m.length; i++) {
			for (int j = 0; j < m[0].length; j++) {
				max = Math.max(max, m[i][j]);
			}
		}
	}

	public BufferedImage createImage() {
		if (bi != null)
			return bi;
		bi = new BufferedImage(m.length*2, m[0].length*2, ColorSpace.TYPE_RGB);
		for (int i = 0; i < m.length; i++)
			for (int j = 0; j < m[0].length; j++) {
				int v = 255 - (int) Math.round(255 * Math.sqrt(m[i][j])/Math.sqrt(max));
				bi.setRGB(2*i, 2*j, (red | v) | (v << 8));
				bi.setRGB(2*i+1, 2*j+1, (red | v) | (v << 8));
				bi.setRGB(2*i+1, 2*j, (red | v) | (v << 8));
				bi.setRGB(2*i, 2*j+1, (red | v) | (v << 8));
			}
		return bi;
	}

	public BufferedImage addFootprints(List<Footprint> name) {
		BufferedImage image = createImage();
		for (Footprint footprint : name) {
			Graphics graphics = image.getGraphics();
			graphics.setColor(Color.BLACK);
			graphics.drawLine(footprint.cell_i, footprint.cell_j, footprint.to_i, footprint.to_j);
		}
		return image;
	}
}
