package app.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingWorker;
import javax.swing.border.Border;

import logic.alignment.AlignmentPrepared;
import logic.alignment.AlignmentPrepared.Footprint;
import logic.tfbs.TFPattern;
import app.BackgroundSequenceManager;
import app.Registry;
import app.SequenceManager;
import app.Helpers.SequencePair;
import app.exception.ConfigurationException;
import app.graphics.HeatMapPanel;
import app.graphics.Image;
import app.intf.Experiment;
import app.intf.Parameters;

public class Billboard {
	private JFrame mainFrame;
	private JTextArea resultArea;
	private JComboBox factorsChooserBox;
	//private JPanel test = new MyPanel();
	private ExperimentPerformer ep = null;
	private DefaultListModel results = null;
	private Border defaultBorder = BorderFactory.createEmptyBorder(5, 5, 5, 5);
	private List<Parameters> pList;
	private BackgroundSequenceManager backgroundSequenceManager;
	private SequenceManager sequenceManager;
	private List<SequencePair> pairs;
	
	
    public Billboard(List<Parameters> ps, SequenceManager sequenceManager, BackgroundSequenceManager backgroundSequenceManager, List<SequencePair> pairs) {
    	this.pList = ps;
    	this.sequenceManager = sequenceManager;
    	this.backgroundSequenceManager = backgroundSequenceManager;
    	this.pairs = pairs;
	}

	/**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private void createAndShowGUI() {
        //Create and set up the window.
    	JMenuBar menuBar = new JMenuBar();
    	JMenu menu = new JMenu("File");
    	menu.setMnemonic(KeyEvent.VK_A);
    	menuBar.add(menu);
    	JMenuItem item = new JMenuItem("Exit", KeyEvent.VK_X);
    	item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
    	});
    	menu.add(item);
        mainFrame = new JFrame("HelloWorldSwing");
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mainFrame.setMinimumSize(new Dimension(640, 480));
        mainFrame.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());

        //Add the ubiquitous "Hello World" label.
        JLabel label = new JLabel("Hello World");
        mainFrame.setJMenuBar(menuBar);
        
        final JList list = new JList(); //data has type Object[]
        list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        list.setLayoutOrientation(JList.HORIZONTAL_WRAP);
        list.setVisibleRowCount(-1);
        final JPopupMenu popup = new JPopupMenu();
        JMenuItem showPssm = new JMenuItem("Show PSSM");
        showPssm.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				System.out.println(e.getSource());
				TFPattern p = (TFPattern) list.getSelectedValue();
				System.out.println(p.getPSSM());
			}
        	
        });
        popup.add(showPssm);
        list.setComponentPopupMenu(popup);

        final TFListModel tfModel = new TFListModel();
        list.setModel(tfModel);
        JScrollPane listScroller = new JScrollPane(list);
        listScroller.setPreferredSize(new Dimension(250, 80));
        listScroller.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), defaultBorder));
        
        JPanel listPanel = new JPanel(new BorderLayout());
        listPanel.setBorder(BorderFactory.createTitledBorder("Transcription factors"));
        listPanel.add(listScroller, BorderLayout.CENTER);
        
        factorsChooserBox = new JComboBox();
        DefaultComboBoxModel model = new DefaultComboBoxModel(Registry.getRegistry().getPatternSetNames().toArray(new String[0]));
        factorsChooserBox.setModel(model);
        factorsChooserBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String setName = (String) factorsChooserBox.getModel().getSelectedItem();
				tfModel.setSetName(setName);
			}
        });
        factorsChooserBox.setBorder(defaultBorder);
        tfModel.setSetName((String) factorsChooserBox.getModel().getSelectedItem());
        
        listPanel.add(factorsChooserBox, BorderLayout.NORTH);
        
        mainFrame.getContentPane().add(label);
        mainFrame.getContentPane().add(listPanel, BorderLayout.WEST);
        mainFrame.getContentPane().add(rightPanel(), BorderLayout.EAST);
        mainFrame.getContentPane().add(centerPanel(), BorderLayout.CENTER);
        
        //test
        
        

        //Display the window.
        
        mainFrame.pack();
        mainFrame.setVisible(true);
    }

   private Component centerPanel() {
	   JPanel center = new JPanel(new BorderLayout());
	   JPanel scrollingArea = new JPanel(new BorderLayout());
	   resultArea = new JTextArea();
	   JScrollPane resultScroll = new JScrollPane(resultArea);
	   scrollingArea.add(resultScroll, BorderLayout.CENTER);
	   center.add(scrollingArea, BorderLayout.CENTER);
	   //center.add(test, BorderLayout.SOUTH);
	   return center;
	}

 private Component rightPanel() {
    	JPanel right = new JPanel();
    	right.setLayout(new BorderLayout(5, 5));
    	
    	results = new DefaultListModel();
    	final JList list = new JList(results);
    	list.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					AlignmentPrepared ap = (AlignmentPrepared) list.getSelectedValue();
					final HeatMapPanel hmp = new HeatMapPanel(new Image(ap.getScoreMatrix().getResultMatrix(), ap.getScoreMatrix().getSMatrix()), ap.getParameters());
					final JFrame frame = new JFrame("heatmap");
					
					Dimension size = hmp.getSize();
					Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
					frame.getContentPane().setLayout(new BorderLayout());
					if (size.height > screenSize.height || size.width > screenSize.width) {
						JScrollPane sp = new JScrollPane(hmp, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
						
						frame.getContentPane().add(sp, BorderLayout.CENTER);
						frame.pack();
						Insets insets = frame.getInsets();
						frame.setSize(screenSize);
						if (size.width < screenSize.width) {
							Dimension d = new Dimension(size.width + insets.left + insets.right, screenSize.height);
							frame.setSize(d);
						}
					} else {
						frame.getContentPane().add(hmp, BorderLayout.CENTER);
						frame.pack();
						Insets insets = frame.getInsets();
						Dimension fdim = new Dimension(size.width + insets.left + insets.right, size.height + insets.top + insets.bottom);
						frame.setSize(fdim);
						frame.setResizable(false);
					}
					frame.addWindowStateListener(new WindowStateListener() {

						public void windowStateChanged(WindowEvent e) {
							if (e.getID() == WindowEvent.COMPONENT_RESIZED) {
								Insets i = frame.getInsets();
								if (frame.getWidth() > hmp.getWidth() + i.left + i.right)
									frame.setSize(hmp.getWidth() + i.left + i.right, frame.getHeight());
							}
						}

						
					});
					frame.setVisible(true);
				}
			}
    	});
    	JScrollPane jsp = new JScrollPane(list);
    	jsp.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), defaultBorder));
    	right.add(jsp, BorderLayout.CENTER);
    	final JButton cancel = new JButton("Cancel");
    	cancel.setEnabled(false);
    	cancel.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (ep != null)
					ep.cancel(true);
			}
    		
    	});
    	JButton go = new JButton("Perform experiment");
    	go.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					ep = new ExperimentPerformer(cancel, pList, Billboard.this);
					ep.execute();
					cancel.setEnabled(true);
				} catch (ConfigurationException e1) {
					showErrorDialog(e1);
				}
			}
    		
    	});
    	JPanel buttonPanel = new JPanel();
    	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    	buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
    	buttonPanel.add(Box.createHorizontalGlue());
    	buttonPanel.add(go);
    	buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
    	buttonPanel.add(cancel);
    	right.add(buttonPanel, BorderLayout.SOUTH);
    	return right;
	}
    
    static class ExperimentPerformer extends SwingWorker<String, AlignmentPrepared> {
    	private Experiment e;
		private JButton cancel;
		private Billboard b;
    	ExperimentPerformer(JButton cancel, List<Parameters> list, Billboard b) throws ConfigurationException {
    		super();
    		this.cancel = cancel;
    		this.e = new Experiment(list, (String) b.factorsChooserBox.getModel().getSelectedItem(), b.pairs, b.sequenceManager, b.backgroundSequenceManager) {
    			@Override
    			public void process(AlignmentPrepared ap, List<Footprint> f) {
    				super.process(ap, f);
    				publish(ap);
    			}
    		};
    		this.b = b;
    	}
		@Override
		protected String doInBackground() throws Exception {
			try {
				e.perform();
				return "";
			} catch (Exception ex) {
				b.showErrorDialog(ex);
				throw ex;
			}
		}
		@Override
		protected void done() {
			cancel.setEnabled(false);
			if (isCancelled())
				b.resultArea.append("Cancelled by the user.");
			else {
				try {
					b.resultArea.append(get() + "\n");
				} catch (InterruptedException e) {
				} catch (ExecutionException e) {
					e.printStackTrace();
				}
			}
		}
		@Override
		protected void process(List<AlignmentPrepared> chunks) {
			super.process(chunks);
			for (AlignmentPrepared prepared : chunks) {
				b.results.addElement(prepared);
				List<Footprint> clusters = prepared.extractClustersDifferently();
				for (Footprint footprint : clusters) {
					b.resultArea.append(footprint.gffString() + "\n");
				}
			}
		}
    }

	protected void showErrorDialog(Exception e) {
		JOptionPane.showMessageDialog(mainFrame,
			    e.getStackTrace(),
			    e.getMessage(),
			    JOptionPane.ERROR_MESSAGE);
	}

	public void start() {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
	}
}
