package app.intf;

import java.util.Properties;

import logic.tfbs.impl.Threshold;
import app.Config.PropName;
import app.exception.BillboardException;
import app.exception.ConfigurationException;

public final class Parameters {
	private final double alpha;
	private final double beta;
	private final double gamma;
	private final int frame;
	private final int jump;
	private final Threshold tt;
	private final int nThreads;
	
	public Parameters(Properties props, double alpha, double beta, double gamma, int frame, int jump) throws ConfigurationException {
		this.nThreads = Integer.parseInt(props.getProperty(PropName.THREADS, "1"));
		this.tt = Threshold.getThreshold(props.getProperty(PropName.THRESHOLD_TYPE));
		if (this.tt.isAbsolute())
			tt.asAbsolute().setThreshold(Double.parseDouble(props.getProperty(PropName.PSSM_THRESHOLD)));
		this.alpha = alpha;
		this.beta = beta;
		this.gamma = gamma;
		this.frame = frame;
		this.jump = jump;
	}
	
	public int getThreadNo() {
		return nThreads;
	}
	
	public Threshold getThresholdType() {
		return tt;
	}
	
	public double getAlpha() {
		return alpha;
	}

	public double getBeta() {
		return beta;
	}

	public int getFrame() {
		return frame;
	}

	public int getJump() {
		return jump;
	}
	
	public String exportString() {
		StringBuffer sb = new StringBuffer();
		sb.append(this.alpha);
	 	sb.append("-");
		sb.append(this.beta);
		sb.append("-");
		sb.append(this.gamma); sb.append("-");
		sb.append(this.frame);
		sb.append("-");
		sb.append(this.jump);
		sb.append("-").append(this.tt);
		return sb.toString();
	}

	public String toString() {
		return exportString();
	}

	public double getGamma() {
		return gamma;
	}

}
