package app.test;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class ArgumentParser {
	private Map<String, String> props = new HashMap<String, String>();
	public ArgumentParser(String args[]) throws ArgumentParserException {
		parse(args);
	}
	
	public ArgumentParser(String[] args, String[] requiredParameters) throws ArgumentParserException {
		this(args);
		checkProperties(requiredParameters);
	}

	private void checkProperties(String[] requiredParameters) throws ArgumentParserException {
		for (String param : requiredParameters) {
			if (props.get(param) == null)
				throw new ArgumentParserException("Required parameter '" + param + "' not specified.");
		}
	}

	private void parse(String[] args) throws ArgumentParserException {
		for (String arg : args) {
			if (!arg.contains("="))
				throw new ArgumentParserException(arg);
			String[] strings = arg.split("=");
			if (strings.length != 2)
				throw new ArgumentParserException(arg);
			props.put(strings[0], strings[1]);
		}
	}
	
	public class ArgumentParserException extends Exception {
		/**
		 * 
		 */
		private static final long serialVersionUID = -619124225799992958L;

		public ArgumentParserException(String argument) {
			super(argument);
		}
	}
	
	public String getArgument(String key) {
		return props.get(key);
	}
	
	public Map<String, String> getAll() {
		return Collections.unmodifiableMap(props);
	}
}
