package app.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.sql.SQLException;

import logic.sequence.io.EnsemblDBReader;
import logic.sequence.io.db.QueryException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.BasicConfigurator;

import app.ensembl.IdNotRecognizedException;
import app.exception.BillboardException;
import app.test.ArgumentParser.ArgumentParserException;
import db.h2.StatisticsPusher;

public class DB {
	private static final Log l = LogFactory.getLog(DB.class);
	private static EnsemblDBReader edbr;
	static {
		try {
			edbr = EnsemblDBReader.getReader();
		} catch (BillboardException e) {
			l.fatal(e);
			System.exit(1);
		}
	}
	private static File inputDir;
	private static StatisticsPusher sp = new StatisticsPusher();
	
	public static void main(String args[]) {
		BasicConfigurator.configure();
		ArgumentParser ap;
		try {
			ap = new ArgumentParser(args, new String[] { "inputDirectory" });
		} catch (ArgumentParserException e) {
			l.error("Could not parse program arguments", e);
			return;
		}
		inputDir = new File(ap.getArgument("inputDirectory"));
		File[] files = inputDir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return !pathname.isHidden() && !pathname.isDirectory();
			}
		});
		long cur = System.currentTimeMillis();
		for (int i = 0; i < files.length; i++) {
			l.info(i + "/" + files.length + ". [Est: " + ((System.currentTimeMillis() - cur) * (files.length - i - 1) / (i + 1)/1000/60)
					+ " minutes]");
			File res = files[i];
			try {
				parse(new BufferedReader(new FileReader(res)));
			} catch (Exception e) {
				l.fatal("Say, whaaaaat???", e);
			}
		}
	}

	private static void parse(BufferedReader bufferedReader) throws IOException, SQLException, QueryException {
		String line = null;
		boolean pairAdded = false;
		while ((line = bufferedReader.readLine()) != null) {
			if (line.equals(""))
				continue;
			String[] split = line.trim().split("\t");
			String hId = split[0];
			String mId = split[1];
			double score = Double.parseDouble(split[2]);
			double spv = Double.parseDouble(split[3]);
			double scorepv = Double.parseDouble(split[4]);
			int starts = -10000 + Integer.parseInt(split[5]);
			int ends = -10000 + Integer.parseInt(split[6]);
			int startsref = -10000 +Integer.parseInt(split[7]);
			int endsref = -10000 + Integer.parseInt(split[8]);
			String hgId;
			try {
				hgId = edbr.getGeneStableIdByTranscriptStableId(hId);
			} catch (IdNotRecognizedException e) {
				e.printStackTrace();
				continue;
			}
			if (!pairAdded) {
				sp.addTranscript(hId, mId, hgId);
				pairAdded = true;
			}
			long id = sp.addCrm(hId, starts, ends, startsref, endsref, score, spv, scorepv);
			line = bufferedReader.readLine();
			StringReader sr = new StringReader(line);
			sr.read(); // read the first '['
			do {
				Occurence o = parseOccurence(sr);
				sp.addOccurence(id, o.name, o.score, o.start, o.strand);
			} while (sr.read() != ']' && sr.read() == ' ');
			line = bufferedReader.readLine();
		}
	}

	private static class Occurence {
		public final String name;
		public final double score;
		public final int start;
		public boolean strand;
		public Occurence(String name, double score, int start, boolean strand) {
			super();
			this.name = name;
			this.score = score;
			this.start = start;
			this.strand = strand;
		}
	}
	
	private static Occurence parseOccurence(StringReader sr) throws IOException {
		sr.read(); // read '['
		String s[] = readUntil(']', sr).split(" ");
		String name = s[0];
		double score = Double.parseDouble(s[1].split(":")[1]);
		int pos = -10000 + Integer.parseInt(s[2].split(":")[1]);
		boolean strand = s[3].split(":")[1].equals("+");
		return new Occurence(name, score, pos, strand);
	}

	private static String readUntil(char c, StringReader sr) throws IOException {
		StringBuilder sb = new StringBuilder();
		char r;
		while ((r = (char) sr.read()) != c) {
			sb.append(r);
		}
		return sb.toString();
	}
}
