package app.test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.SQLException;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.BasicConfigurator;

import logic.sequence.io.EnsemblDBReader;
import logic.sequence.io.db.QueryException;
import app.exception.BillboardException;

public class Grouper {
	private static final Log l = LogFactory.getLog(Grouper.class);

	/**
	 * @param args
	 */
	
	private static final File dir = new File("wyniki");
	private static final File res = new File(dir, "podzielone");
	private static final File hGenes = new File("hgenes");
	private static final File rejected = new File(dir, "odrzuty");
	
	public static void main(String[] args) throws IOException, SQLException, QueryException, BillboardException {
		BasicConfigurator.configure();
		Set<String> hGeneIdentifiers = readAllGenes(hGenes);
		EnsemblDBReader edbr = EnsemblDBReader.getReader();
		File[] gffFiles = dir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isFile() && pathname.getName().endsWith((".gff"));
			}
		});
		int ileTranskryptow = 0;
		for (File f : gffFiles) {
			l.info("Processing file " + f.getName());
			BufferedReader br = new BufferedReader(new FileReader(f));
			String line = null, lastId = null, lastHomolId = null;
			BufferedWriter bw = null;
			while ((line = br.readLine()) != null) {
				String homoStableTranscriptId = line.split("\t")[0];
				if (!homoStableTranscriptId.equals(lastId)) {
					ileTranskryptow++;
					lastId = homoStableTranscriptId;
					String homoId = 
						edbr.getGeneStableIdByTranscriptStableId(homoStableTranscriptId);
					if (homoId.equals(lastHomolId)) {
						
					} else {
						lastHomolId = homoId;
						hGeneIdentifiers.remove(lastHomolId);
						if (bw != null) {
							bw.flush(); bw.close();
						}
						//bw = new BufferedWriter(new FileWriter(new File(res, lastHomolId + ".gff")));
					}
					//bw.write(line); bw.newLine();
				}
			}
			//bw.flush(); bw.close();
		}
		System.err.println(ileTranskryptow);
		//dumpRejected(hGeneIdentifiers);
	}
	
	private static void dumpRejected(Set<String> geneIdentifiers) throws IOException {
		BufferedWriter bw = new BufferedWriter(new FileWriter(rejected));
		for (String id : geneIdentifiers) {
			bw.write(id);
			bw.newLine();
		}
		bw.flush(); bw.close();
	}

	private static Set<String> readAllGenes(File f) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(f));
		Set<String> result = new HashSet<String>();
		String line;
		while ((line = br.readLine()) != null) {
			result.add(line.trim());
		}
		return result;
	}

}
