package db.h2;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.h2.tools.RunScript;

public class H2Connection {
	private static Log l = LogFactory.getLog(H2Connection.class);
	static {
		try {
			Class.forName("org.h2.Driver");
		} catch (ClassNotFoundException e) {
			l.fatal("Could not find H2 database driver.");
			System.exit(1);
		}
	}
	
	private static final String cUrl = "jdbc:h2:file:db/stats;FILE_LOCK=NO;SCHEMA=PUBLIC";
	private static final String ifExists = ";IFEXISTS=TRUE";
	private static final String schemaFile = "db_scripts/create.sql";
	
	private Connection c = null;
	
	private static H2Connection instance = new H2Connection();
	
	public static H2Connection getConnection() {
		return instance;
	}
	
	private synchronized boolean check() throws SQLException {
		if (c == null || c.isClosed()) {
			try {
				c = DriverManager.getConnection(cUrl + ifExists, "sa", "");
				l.info("Using previous storage");
			} catch (SQLException e) {
				l.info("First time storage");
				c = DriverManager.getConnection(cUrl, "sa", "");
				try {
					RunScript.execute(c, new BufferedReader(new FileReader(schemaFile)));
					l.info("Storage created");
				} catch (FileNotFoundException e1) {
					l.fatal("Failed to create storage");
					return false;
				}
			}
		}
		return true;
	}
	
	public void executeUpdate(String sql) throws SQLException {
		if (check()) {
			Statement s = c.createStatement();
			s.executeUpdate(sql);
		}
	}
	
	public ResultSet executeQuery(String sql) throws SQLException {
		if (check()) {
			Statement s = c.createStatement();
			return s.executeQuery(sql);
		} else {
			return null;
		}
	}
	
	public Object unique(String sql) throws SQLException {
		ResultSet r = executeQuery(sql);
		if (r == null || !r.first()) {
			return null;
		} else {
			return r.getObject(1);
		}
	}
	
	public Long getIdentity() throws SQLException {
		return (Long) unique("SELECT IDENTITY();");
	}
}
