package logic.multiset;

import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.LinkedList;

import logic.sequence.TFSite;
import logic.tfbs.TFPattern;
import logic.utils.ArrayView;

public class MultiSet {

	private int size;
	private TFPattern[] patterns;
	private TFSite[][] sites;
	
	public MultiSet(ArrayView<Collection<TFSite>> arr) {
		LinkedList<TFSite> ll = new LinkedList<TFSite>();
		for (int i = 0; i < arr.length(); i++) {
			Collection<TFSite> c = arr.getAt(i);
			for (TFSite site : c)
					ll.add(site);
		}
		TFSite[] ar = ll.toArray(new TFSite[0]);
		Arrays.sort(ar, new Comparator<TFSite>() {
			public int compare(TFSite o1, TFSite o2) {
				if (o1.getPattern().getId() == o2.getPattern().getId())
					return (o1.getScore() < o2.getScore()) ? 1 : -1;
				else
					return o1.getPattern().getId() - o2.getPattern().getId();
			}
		});
		this.size = ar.length;

		if (size > 0) {
			LinkedList<TFPattern> patterns = new LinkedList<TFPattern>();
			LinkedList<LinkedList<TFSite>> sites = new LinkedList<LinkedList<TFSite>>();
			
			int lastId = Integer.MIN_VALUE;
			for (int i = 0; i < ar.length; i++) {
				TFSite cur = ar[i];
				if (cur.getPattern().getId() == lastId) {
					sites.getLast().add(cur);
				} else {
					patterns.add(cur.getPattern());
					LinkedList<TFSite> s = new LinkedList<TFSite>();
					s.add(cur);
					sites.add(s);
					lastId= cur.getPattern().getId();
				}
			}
			
			this.patterns = patterns.toArray(new TFPattern[0]);
			this.sites = new TFSite[this.patterns.length][];
			
			int i = 0;
			for (LinkedList<TFSite> s : sites) {
				TFSite[] tfsa = s.toArray(new TFSite[0]);
				this.sites[i++] = tfsa;
			}
		}
	}
	
	/**
	 * F(S1, S2) = ALPHA * |cut(S1, S2)| - BETA * |simdiff(S1, S2)| - GAMMA
	 * @param other
	 * @return the score or similarity between multi-sets according to the formula
	 */
	public CutAndDiff compare(MultiSet other) {
		int t_i = 0, o_i = 0;
		int cut = 0;
		if (other.size > 0 && this.size > 0)
			while ((t_i < this.patterns.length) && (o_i < other.patterns.length)) {
				if (this.patterns[t_i].getId() == other.patterns[o_i].getId()) {
					cut += Math.min(this.sites[t_i].length,
							other.sites[o_i].length);
					t_i++; o_i++;
				} else if (this.patterns[t_i].getId() < other.patterns[o_i].getId()) {
					t_i++;
				} else {
					o_i++;
				}
			}
		return new CutAndDiff(cut, other.size + this.size - 2*cut);
	}
	
	public class CutAndDiff {
		public final int cut, diff;
		public CutAndDiff(int cut, int diff) {
			this.cut = cut;
			this.diff = diff;
		}
	}

}
