package logic.sequence.impl;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import logic.sequence.DNASequence;
import logic.sequence.TFSite;
import logic.tfbs.PatternSet;
import logic.tfbs.impl.NoSuchLetter;
import logic.tfbs.impl.Threshold;

public class PreparedSequence implements DNASequence {
	private DNASequence delegate;
	private Collection<TFSite>[] prepared;
	private PatternSet patternSet;
	private Threshold threshold;
	
	public PreparedSequence(DNASequence s, PatternSet patternSet, Threshold threshold) {
		this.delegate = s;
		this.patternSet = patternSet;
		this.threshold = threshold;
	}

	public char getAt(int i) {
		return delegate.getAt(i);
	}

	public String getFragment(int offset, int length) {
		return delegate.getFragment(offset, length);
	}

	public String getName() {
		return delegate.getName();
	}

	public int length() {
		return delegate.length();
	}

	public Collection<TFSite>[] getPatterns() {
		if (prepared == null) {
			if (threshold.isAbsolute())
				prepared = delegate.matchPatterns(
						patternSet, threshold.asAbsolute().getThreshold());
			else
				prepared = delegate.matchPatterns(patternSet);
		}
		return prepared;
	}
	
	public Collection<TFSite>[] getPatterns(double threshold) throws NoSuchLetter {
		if (prepared == null)
			prepared = delegate.matchPatterns(patternSet, threshold);
		return prepared;
	}

	
	public Collection<TFSite>[] getSites() {
		return prepared;
	}
	
	public Collection<TFSite> getSites(int pos, int offset) {
		List<TFSite> result = new LinkedList<TFSite>();
		for (int i = pos; i < pos + offset + 1; i++) {
			result.addAll(prepared[i]);
		}
		return result;
	}
	
	public Collection<TFSite>[] matchPatterns(PatternSet patternSet, double threshold) {
		throw new UnsupportedOperationException("Can't be invoked on a prepared sequence");
	}
	
	public Collection<TFSite>[] matchPatterns(PatternSet patternSet) {
		throw new UnsupportedOperationException("Can't be invoked on a prepared sequence");
	}

	public void exportFastaStream(OutputStream os) throws IOException {
		delegate.exportFastaStream(os);
	}

	public void exportFastaStream(BufferedWriter w) throws IOException {
		delegate.exportFastaStream(w);
	}
}
