package logic.sequence.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import logic.sequence.DNASequence;
import logic.sequence.impl.BadAlphabet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import app.ensembl.IdNotRecognizedException;
import app.ensembl.Species;

public class EnsembleWSReader {
	protected final Log logger = LogFactory.getLog(getClass());
	
	/**
	 * <b>Warning!</b> The order of ids is not the same as the order of the result!
	 * @param ids
	 * @param promoter
	 * @param dataSet
	 * @param length
	 * @return
	 * @throws IOException
	 * @throws BadAlphabet
	 */
	
	public DNASequence[] getEnsemblSeqById(String[] ids, boolean promoter, Species species, int beforeStart, int afterStart)
		throws IOException, BadAlphabet {
		String bs = Integer.toString(beforeStart);
		String[] atts = new String[] {"gene_stable_id", "gene_exon_intron"};
		String[] fns = new String[] {"upstream_flank", "ensembl_gene_id"};
		ArrayList<DNASequence> al = new ArrayList<DNASequence>();
		String[][] splits = split(ids);
		for (String[] split : splits) {
			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < split.length; i++) {
				if (i > 0)
					sb.append(",");
				sb.append(split[i]);
			}
			String[] fvs = new String[] {bs, sb.toString()};
			String query = query(getDatasetString(species.getDataSet()) + getAttributeString(atts) + getFiltersString(fns, fvs));
			URL u = new URL(serviceUrl + URLEncoder.encode(query, "UTF-8"));
			System.out.println(u);
			al.addAll(FastaSequenceReader.read(u.openStream(), true, beforeStart + afterStart));
		}
		return al.toArray(new DNASequence[0]);
	}
	
	
	/**
	 * 
	 * @param id
	 * @param promoter
	 * @param beforeStart
	 * @param afterStart
	 * @return DNASequence or null if no sequence with this id was found
	 * @throws IOException
	 * @throws BadAlphabet
	 * @throws IdNotRecognizedException 
	 */
	public DNASequence getEnsembleSequenceById(String id, boolean promoter, int beforeStart, int afterStart) throws IOException, BadAlphabet {
		Species s;
		try {
			s = Species.getForEnsembleId(id);
		} catch (IdNotRecognizedException e) {
			e.printStackTrace();
			return null;
		}
		DNASequence[] ensemblSeqById = getEnsemblSeqById(new String[] { id }, promoter, s, beforeStart, afterStart);
		if (ensemblSeqById.length > 0)
			return ensemblSeqById[0];
		else
			return null;
	}
	
	/**
	 * 
	 * @param ensembleID
	 * @param species - the species where the homologs should be found
	 * @return
	 * @throws IOException 
	 * @throws IdNotRecognizedException 
	 */
	public String[] getHomologs(String ensembleId, Species species) throws IOException, IdNotRecognizedException {
		String query = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><!DOCTYPE Query><Query  virtualSchemaName = \"default\" formatter = \"TSV\" header = \"0\" uniqueRows = \"0\" count = \"\" datasetConfigVersion = \"0.6\" >"
			+ "<Dataset name = \"" + Species.getForEnsembleId(ensembleId).getDataSet() + "\" interface = \"default\" >"
			+ "<Filter name = \"ensembl_gene_id\" value = \"" + ensembleId + "\"/>"
			+ "<Attribute name = \"ensembl_gene_id\" />"
			+ "<Attribute name = \"" + species.asAttribute() + "\" />"
			+ "</Dataset></Query>";
		URL u = new URL(serviceUrl + URLEncoder.encode(query, "UTF-8"));
		System.out.println(u);
		BufferedReader br = new BufferedReader(new InputStreamReader(u.openStream()));
		String line = null;
		List<String> l = new LinkedList<String>();
		while ((line = br.readLine()) != null) {
			String[] tokens = line.split("\t");
			l.add(tokens[1].trim());
		}
		return l.toArray(new String[0]);
	}
	
	
	
	private String[][] split(String[] ids) {
		List<String[]> splits = new LinkedList<String[]>();
		int n = ids.length;
		while (n >= 20) {
			String[] r = new String[20];
			for (int i = 0; i < 20; i ++) {
				n--;
				r[i] = ids[n];
			}
			splits.add(r);
			}
		if (n > 0) {
			String[] r = new String[n];
			for (int i = 0; i < n; i++) {
				r[i] = ids[i];
			}
			splits.add(r);
		}
		return splits.toArray(new String[0][]);	
	}

	private final String serviceUrl = "http://www.biomart.org/biomart/martservice?query=";
	
	private String query(String content) { 
		return "<Query  virtualSchemaName = \"default\" formatter = \"FASTA\" header = \"0\" uniqueRows = \"0\" count = \"\" datasetConfigVersion = \"0.6\" >"
		+ content + "</Dataset></Query>";
	}
	
	private String getDatasetString(String datasetName) {
		String n = datasetName;
		if (n == null)
			n = "hsapiens_gene_ensembl";
		return "<Dataset name = \"" + n + "\" interface = \"default\" >";
	}
	
	private String getAttributeString(String[] attributes) {
		String[] atts = attributes;
		if (atts == null)
			atts = new String[] {"gene_stable_id_v",  "gene_stable_id", "gene_flank"};
		StringBuilder sb = new StringBuilder();
		for (String el : atts) {
			sb.append("<Attribute name = \"");
			sb.append(el);
			sb.append("\" />");
		}
		return sb.toString();
	}
	
	private String getFiltersString(String[] names, String[] values) {
		String[] ns = names;
		String[] vs = values;
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < ns.length; i++) {
			sb.append("<Filter name = \"");
			sb.append(ns[i]);
			sb.append("\" value = \"");
			sb.append(vs[i]);
			sb.append("\"/>");
		}
		return sb.toString();
	}
	
}
