package logic.sequence.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import logic.sequence.DNASequence;
import logic.sequence.impl.BadAlphabet;
import app.Config;

public class FastaSequenceReader {
	private static Logger l = Logger.getLogger(FastaSequenceReader.class.getName());
	
	public static Collection<DNASequence> readDNASeqFromFile(File f, boolean acceptAmbigous) throws IOException, BadAlphabet {
		return read(new FileInputStream(f), acceptAmbigous, -1);
	}
	
	public static Collection<DNASequence> read(InputStream is, boolean acceptAmbigous, int limit) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		List<DNASequence> result = new LinkedList<DNASequence>();
		String line;
		String seqName = null;
		line = br.readLine();
		while (line != null) {
			if (line.startsWith(">")) {
				seqName = line.substring(1).trim();
				StringBuilder seq = new StringBuilder();
				while ((line = br.readLine()) != null && !line.startsWith(">")) {
					seq.append(line.trim());
				}
				try {
					String string = seq.toString().trim().toLowerCase();
					if (limit >= 0)
						if (string.length() > limit)
							string = string.substring(0, limit);
					result.add(Config.seqb.createDNASequence(seqName, string, acceptAmbigous));
				} catch (BadAlphabet e) {
					l.log(Level.WARNING, "Could not create sequence " + seqName, e);
				}
			} else {
				line = br.readLine();
			}
		};
/*		boolean was = false;
		while ((line = br.readLine()) != null) {
			if (line.startsWith(">")) {
				if (was) {
					try {
						result.add(Config.seqb.createDNASequence(seqName, sequence.toString().trim().toLowerCase(), true));
					} catch (BadAlphabet e) {
						l.log(Level.WARNING, "Could not create sequence " + seqName, e);
					}
					sequence = new StringBuilder();
				}
				was = true;
				seqName = line.substring(1).trim();
			} else {
				sequence.append(line);
			}
		}
		try {
			result.add(Config.seqb.createDNASequence(seqName, sequence.toString().trim().toLowerCase(), true));
		} catch (BadAlphabet e) {
			l.log(Level.WARNING, "Could not create sequence " + seqName, e);
		}*/
		return result;
	}
	
	public static void writeSequenceToDir(DNASequence s, File destDir) throws IOException {
		if (!destDir.isDirectory()) {
			throw new IllegalArgumentException(destDir.getPath() + " is not a directory!");
		}
		File result = new File(destDir, s.getName());
		BufferedWriter bw = new BufferedWriter(new FileWriter(result));
		bw.write(">" + s.getName() + "\n");
		int line = 0, lineLength = 80;
		for (int i = 0; i < s.length(); i++, line++) {
			if (line == lineLength) {
				bw.write("\n");
				line = 0;
			}
			bw.write(s.getAt(i));
		}
		bw.close();
	}
}
