package logic.sequence.io.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import logic.sequence.io.db.sql.entity.manager.EntityManager;
import app.exception.BillboardException;


public class EnsemblConnection {
	private static final Log l = LogFactory.getLog(EnsemblConnection.class); 
	
	public static final String host = "jdbc:mysql://ensembldb.ensembl.org";
	public static final String user = "anonymous";
	private Connection connection;
	private static EnsemblConnection ec = null;
	
	private EnsemblConnection() throws SQLException {
		this.connection = doGetConnection();
	}
	
	public synchronized static EnsemblConnection getConnection() throws BillboardException {
		if (ec == null) {
			try {
				Class.forName("com.mysql.jdbc.Driver");
				ec = new EnsemblConnection();
			} catch (Exception e) {
				throw new BillboardException("Could not create database connection", e);
			}
		}
		return ec;
	}
	
	private Connection doGetConnection() throws SQLException {
		return DriverManager.getConnection(host, user, "");
	}
	
	public Object uniqueResult(String sql, EntityManager<?> em) throws SQLException, QueryException {
		checkConnection();
		Statement st = connection.createStatement();
		try {
			ResultSet set = st.executeQuery(sql);
			if (!set.next()) {
				throw new QueryException("No row fits the criteria: " + sql);
			}
			Object entity = em.createFromResultSet(set);
			if (set.next())
				throw new QueryException("More than one row fits the criteria: " + sql);
			return entity;
		} finally {
			try {
				st.close();
			} catch (Exception e) {
				l.warn("Error closing statement", e);
			}
		}
	}
	
	public List<? extends Object> list(String sql, EntityManager<?> em) throws SQLException {
		checkConnection();
		Statement st = connection.createStatement();
		try {
			ResultSet set = st.executeQuery(sql);
			return em.createAllFromResultSet(set);
		} finally {
			try {
				st.close();
			} catch (Exception e) {
				l.warn("Error closing statement", e);
			}
		}
	}

	private void checkConnection() throws SQLException {
		if (connection.isClosed())
			connection = doGetConnection();
	}

	public String uniqueResult(String sql) throws SQLException, QueryException {
		checkConnection();
		Statement st = connection.createStatement();
		try {
			ResultSet set = st.executeQuery(sql);
			if (!set.next()) {
				throw new QueryException("No row fits the criteria: " + sql);
			}
			String entity = set.getString(1);
			if (set.next())
				throw new QueryException("More than one row fits the criteria: " + sql);
			return entity;
		} finally {
			try {
				st.close();
			} catch (Exception e) {
				l.warn("Error closing statement", e);
			}
		}
	}

}
