package logic.sequence.io.db;

import app.ensembl.IdNotRecognizedException;
import app.ensembl.Species;


public class SqlLib {
	
	public static final String test = "select count(*) from " + Schema.HOMO + ".gene;";
	
//	public static final String homoGenes = "select a.gene_id, b.stable_id, seq_region_id, seq_region_start, seq_region_end, seq_region_strand from homo_sapiens_core_47_36i.gene a, homo_sapiens_core_47_36i.gene_stable_id b where a.gene_id = b.gene_id and status = 'KNOWN' and is_current = 1 and biotype = 'protein_coding';";
//	public static final String mouseGenes = "select a.gene_id, b.stable_id, seq_region_id, seq_region_start, seq_region_end, seq_region_strand from " + Schema.getMus("gene") + " a, " + Schema.getMus("gene_stable_id") + " b where a.gene_id = b.gene_id and status = 'KNOWN' and is_current = 1 and biotype = 'protein_coding';";
	public static final String homoChromosomes = "select * from homo_sapiens_core_47_36i.seq_region where coord_system_id = 17;";

	public static String mouseChromosomes = "select * from " + Schema.getMus("seq_region") + " where coord_system_id = 1;";
	
	public static final String getGenesForSpecies(Species s) {
		return "select a.gene_id, b.stable_id, seq_region_id, seq_region_start, seq_region_end, seq_region_strand from "
		+ Schema.getSpecies("gene", s) + " a, "
		+ Schema.getSpecies("gene_stable_id", s)
		+ " b where a.gene_id = b.gene_id and status = 'KNOWN' and is_current = 1 and biotype = 'protein_coding';";
	}
	
	/**
	 * fragment asm_seq_region_id, cmp_seq_region_id, asm_start, asm_end, cmp_start, cmp_end, sequence
	 * @param ch_seq_region
	 * @param start
	 * @param end
	 * @return
	 * @throws IdNotRecognizedException 
	 */
	
	public static final String getTranscriptForStableId(String stableId) throws IdNotRecognizedException {
		Species s = Species.getForEnsembleId(stableId);
		return
		"select a.transcript_id, a.gene_id, a.seq_region_id, a.seq_region_start," +
				" a.seq_region_end, a.seq_region_strand, d.stable_id " +
				"from " + Schema.getSpecies("transcript", s) + " a, " +
				Schema.getSpecies("transcript_stable_id", s) + " d " +
				"where d.transcript_id = a.transcript_id and d.stable_id = '" + stableId + "';";
	}
	
	public static final String getGeneForStableId(String stableId) throws IdNotRecognizedException {
		Species s = Species.getForEnsembleId(stableId);
		return "select a.gene_id, b.stable_id, a.seq_region_id, a.seq_region_start, a.seq_region_end," +
				" a.seq_region_strand " +
				"from " + Schema.getSpecies("gene", s) + " a, " +
						Schema.getSpecies("gene_stable_id", s) + " b " +
				"where a.gene_id = b.gene_id and b.stable_id = '" + stableId + "';";

	}
	
	public static final String getAssemblyFragments(int ch_seq_region, int start, int end, Species s) {
		return "select asm_seq_region_id, cmp_seq_region_id, asm_start, asm_end, cmp_start, cmp_end, sequence from " + Schema.getSpecies("assembly", s) + " a, " + Schema.getSpecies("dna", s) + " b" +
		" where ((asm_start <= " + start + " and asm_end > " + start + ") or (asm_start > " + start + " and asm_start < " + end + "))" +
				" and asm_seq_region_id = " + ch_seq_region + " and a.cmp_seq_region_id = b.seq_region_id;";
	}
	
	public static final String getSequence(String seq_region, int start, int end, Species s) {
		return "select substring(sequence, " + start + ", " + (end - start + 1) + ") from " + Schema.getSpecies("dna", s) + " where seq_region_id = " + seq_region + ";";
	}
	
	/**
	 * Get mouse homologs for this human stable id
	 * @param geneId
	 * @return
	 */
	public static final String getGeneHomologs(String geneStableId) {
		return "select a.stable_id from " + Schema.getCompara("homology_member") + " b, " +
		Schema.getCompara("member") + " a," +
				"(select a.homology_id from " + Schema.getCompara("homology_member") + " a, " +
						Schema.getCompara("homology") + " b, " +
								Schema.getCompara("member") + " c where c.stable_id = '" + 
				geneStableId + "' and c.member_id = a.member_id and a.homology_id = b.homology_id and b.description = 'ortholog_one2one')" +
						"c where a.member_id = b.member_id and b.homology_id = c.homology_id" +
						" and a.genome_db_id = 57 and source_name = 'ENSEMBLGENE';";
	}
	
	public static final String getTranscriptsForStableId(String stableId, Species s) {
		return "select a.transcript_id, a.gene_id, a.seq_region_id, a.seq_region_start," +
				" a.seq_region_end, a.seq_region_strand, d.stable_id " +
				"from " + Schema.getSpecies("transcript", s) + " a, " +
				Schema.getSpecies("gene", s) + " b, " +
				Schema.getSpecies("gene_stable_id", s) + " c, " +
				Schema.getSpecies("transcript_stable_id", s) + " d " +
				"where c.stable_id = '" + stableId + "' and " +
				"c.gene_id = b.gene_id and b.gene_id = a.gene_id and " +
				"a.status = 'KNOWN' and a.is_current = 1 " +
				"and d.transcript_id = a.transcript_id;";
	}
	
	public static final String getFirstExon(int transcriptId, Species s) {
		return "select a.exon_id, a.seq_region_id, a.seq_region_start, a.seq_region_end," +
				" a.seq_region_strand from " + Schema.getSpecies("exon", s) + " a, " +
						Schema.getSpecies("exon_transcript", s) + " b, " +
								Schema.getSpecies("transcript", s) + " c " +
				"where a.is_current = 1 and c.transcript_id = " + transcriptId + " and c.transcript_id = b.transcript_id and b.rank = 1 and b.exon_id = a.exon_id;";
	}

	public static String getDnafrag(String regionName, Species s) {
		return "select dnafrag_id, length, name, genome_db_id, coord_system_name " +
				"from " + Schema.getCompara("dnafrag") +
				" where name = '" + regionName + "' and genome_db_id = " +
				Schema.getGenomeDbId(s) + ";";
	}
	
	public static String getDnafragById(int id) {
			return "select dnafrag_id, length, name, genome_db_id, coord_system_name " +
					"from " + Schema.getCompara("dnafrag") +
					" where dnafrag_id = " + id + ";";
	}

	public static String getAlignedRegion(int dnafragId, int seqRegionStart,
			int seqRegionEnd) {
		return "select genomic_align_id, genomic_align_block_id, " +
				"dnafrag_id, dnafrag_start, dnafrag_end, dnafrag_strand, cigar_line from " +
		Schema.getCompara("genomic_align") + " where "
		+ "dnafrag_id = " + dnafragId + " and method_link_species_set_id = 281 and " +
				"((dnafrag_start <= " + seqRegionStart + " and " +
						"dnafrag_end > " + seqRegionStart + ")" +
				" or (dnafrag_start < " + seqRegionEnd + " and dnafrag_end >= " +
						seqRegionEnd + ")" +
		        " or (dnafrag_start > " + seqRegionStart + " and dnafrag_end < " +
		        seqRegionEnd + "));";
	}
	
	public static String getPairedAlignedRegion(long genomicAlignBlockId, long humanGenomicAlignId) {
		return "select genomic_align_id, genomic_align_block_id, dnafrag_id, " +
				"dnafrag_start, dnafrag_end, dnafrag_strand, cigar_line " +
				"from " + Schema.getCompara("genomic_align") + " " +
				"where genomic_align_block_id = " + genomicAlignBlockId + " " +
				"and genomic_align_id != " +
				humanGenomicAlignId + ";";
	}

	public static String getGeneStableIdForTranscriptStableId(String stableId,
			Species s) {
		return
		"select b.stable_id from " + Schema.getSpecies("gene", s) + " a, " +
		Schema.getSpecies("gene_stable_id", s) + " b, " +
		Schema.getSpecies("transcript", s) + " c, " +
		Schema.getSpecies("transcript_stable_id", s) + " d " +
		"where a.gene_id = b.gene_id and c.gene_id = a.gene_id and c.transcript_id = d.transcript_id " +
		"and d.stable_id = '" + stableId + "';";
	}
	
	public static String getAlignedRegion(int dnafragId, int seqRegionPos) {
		return "select genomic_align_id, genomic_align_block_id, " +
				"dnafrag_id, dnafrag_start, dnafrag_end, dnafrag_strand, cigar_line from " +
		Schema.getCompara("genomic_align") + " where "
		+ "dnafrag_id = " + dnafragId + " and method_link_species_set_id = 281 and " +
				"((dnafrag_start <= " + seqRegionPos + " and " +
						"dnafrag_end >= " + seqRegionPos + "));";
	}
	
	

}
