package logic.tfbs.impl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;

import logic.sequence.Nucleotide;
import logic.tfbs.PSSM;

public class PSSMImpl implements PSSM {
	
	private char[] alphabeth;
	private int width;
	private String name;
	private int[][] matrix;
	private int[] quantity;
	private int id;
	
	public char[] getAlphabeth() {
		return alphabeth;
	}

	public int getAlphabethSize() {
		return alphabeth.length;
	}

	public int getMatrixWidth() {
		return width;
	}

	public int getQuantity(int position) {
		return quantity[position];
	}

	public int getValueAt(int letter, int position) {
		return matrix[letter][position];
	}

	public static class PSSMIOFactory {
		public PSSM buildSimple(Reader r, String name) throws IOException {
			BufferedReader br = new BufferedReader(r);
			String[] A = br.readLine().trim().split(" +");
			String[] C = br.readLine().trim().split(" +");
			String[] G = br.readLine().trim().split(" +");
			String[] T = br.readLine().trim().split(" +");
			String[][] m = new String[][]{A, C, G, T};
			br.close();

			if (!((A.length == T.length) && (T.length == G.length) && (G.length == C.length)))
				throw new IOException("Something is wrong with the matrix");

			PSSMImpl result = new PSSMImpl();
			result.alphabeth = new char[]{'a', 'c', 'g', 't'};
			result.width = A.length;
			result.name = name;
			result.matrix = new int[result.alphabeth.length][result.width];
			for (int i = 0; i < result.alphabeth.length; i++) {
				for (int j = 0; j < result.width; j++) {
					result.matrix[i][j] = Integer.parseInt(m[i][j]);
				}
			}
			result.quantity = computeAndCheckQuantity(result.matrix);
			return result;
		}

		private static int[] computeAndCheckQuantity(int[][] matrix) {
			int[] res = new int[matrix[0].length];
			for (int i = 0; i < matrix[0].length; i++) {
				for (int j = 0; j < matrix.length; j++) {
					res[i] += matrix[j][i];
				}
			}
			return res;
		}
	}

	public String getName() {
		return name;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(name); sb.append("\n");
		for (int i = 0; i < alphabeth.length; i++) {
			sb.append(alphabeth[i] + "\t");
			for (int j = 0; j < width; j++) {
				sb.append(matrix[i][j]); sb.append("\t");
			}
			sb.append("\n");
		}
		return sb.toString();
	}

	public int getId() {
		return id;
	}

	/**
	 * @return the row for this letter or null if the letter not in alphabet
	 */
	public int getRow(char alpha) throws NoSuchLetter {
		for (int i = 0; i < alphabeth.length; i++) {
			if (alphabeth[i] == alpha)
				return i;
		}
		throw new NoSuchLetter(alpha);
	}
	
	public int getComplimentaryRow(char alpha) throws NoSuchLetter {
		return getRow(Nucleotide.getComplimentary(alpha));
	}
}
