package logic.tfbs.impl;

import app.exception.ConfigurationException;

public abstract class Threshold {
	
	public static Threshold getThreshold(String type) throws ConfigurationException {
		if (type.equals("absolute")) {
			return new AbsoluteThreshold();
		} else if (type.equals("balanced")) {
			return new BalancedThreshold(); 
		} else
			throw new ConfigurationException("Unrecognized threshold type.");
	}
	
	public abstract boolean isBalanced();
	public abstract boolean isAbsolute();
	
	public  AbsoluteThreshold asAbsolute() {
		return (AbsoluteThreshold) this;
	}
	public  BalancedThreshold asBalanced() {
		return (BalancedThreshold) this;
	}
	
	public static class BalancedThreshold extends Threshold {

		@Override
		public boolean isAbsolute() {
			return false;
		}

		@Override
		public boolean isBalanced() {
			return true;
		}
		
		public String toString() {
			return "balancedThreshold";
		}
		
	}
	
	public static class AbsoluteThreshold extends Threshold {
		private double threshold;

		@Override
		public boolean isAbsolute() {
			return true;
		}

		@Override
		public boolean isBalanced() {
			return false;
		}

		public double getThreshold() {
			return threshold;
		}

		public void setThreshold(double threshold) {
			this.threshold = threshold;
		}
		
		public String toString() {
			return "absoluteThreshold-" + threshold;
		}
	}
}
