package logic.utils;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Any change of elements in ArrayView changes the underlying array.
 * 
 * @author Mateusz
 *
 * @param <T>
 */
public final class ArrayView<T> implements Iterable<T> {
	private T[] a;
	private int offset;
	private int length;
	public ArrayView(T[] a, int offset, int length) {
		this.a = a;
		this.offset = offset;
		this.length = length;
		if (a.length < offset + length)
			throw new ArrayIndexOutOfBoundsException(offset + length);
	}
	
	final public T getAt(int i) {
		if (i < length)
			return a[offset + i];
		else
			throw new ArrayIndexOutOfBoundsException(i);
	}

	public Iterator<T> iterator() {
		return new Iterator<T>() {
			
			private int cur = 0;

			public boolean hasNext() {
				if (cur < ArrayView.this.length - 1)
					return true;
				return false;
			}

			public T next() {
				if (!hasNext())
					throw new NoSuchElementException();
				return a[offset + cur++];
			}

			public void remove() {
				throw new UnsupportedOperationException("The array view is read-only.");
			}
			
		};
	}
	
	final public int length() {
		return length;
	}
}
