package logic.sequence.io;

import static org.junit.Assert.*;

import java.io.IOException;
import java.util.Arrays;

import logic.sequence.DNASequence;
import logic.sequence.impl.BadAlphabet;

import org.junit.Test;

import app.ensembl.IdNotRecognizedException;
import app.ensembl.Species;

public class EnsembleWSReaderTest {

	@Test
	public void testGetEnsemblSeqById() throws IOException, BadAlphabet {
		EnsembleWSReader ensReader = new EnsembleWSReader();
		String ids[] = new String[] {"ENSG00000122877", "ENSG00000116285", "ENSG00000176697", "ENSG00000198576", "ENSG00000073756", "ENSG00000117525", "ENSG00000106236", "ENSG00000153234", "ENSG00000120738", "ENSG00000119508", "ENSG00000171951", "ENSG00000175416", "ENSG00000118515", "ENSG00000177606", "ENSG00000162772", "ENSG00000128564", "ENSG00000159388", "ENSG00000181195", "ENSG00000164442", "ENSG00000092820"};
		DNASequence[] dnas = ensReader.getEnsemblSeqById(ids, true, Species.HOMO_SAPIENS, 20, 20);
		for (DNASequence sequence : dnas) {
			System.out.println(sequence);
		}
	}

	@Test
	public void testOrder() throws IOException, BadAlphabet {
		EnsembleWSReader ensReader = new EnsembleWSReader();
		String ids[] = new String[] {"ENSG00000122877", "ENSG00000116285", "ENSG00000176697", "ENSG00000198576", "ENSG00000073756", "ENSG00000117525", "ENSG00000106236", "ENSG00000153234", "ENSG00000120738", "ENSG00000119508", "ENSG00000171951", "ENSG00000175416", "ENSG00000118515", "ENSG00000177606", "ENSG00000162772", "ENSG00000128564", "ENSG00000159388", "ENSG00000181195", "ENSG00000164442", "ENSG00000092820"};
		DNASequence[] dnas = ensReader.getEnsemblSeqById(ids, true, Species.HOMO_SAPIENS, 20, 20);
		
		assertTrue(ids.length == dnas.length);
		
		for (int i = 0; i < dnas.length; i++) {
			assertNotNull("" + i, ids[i]);
			assertNotNull("" + i, dnas[i]);
			System.out.println(ids[i] + " " + dnas[i] + " " + dnas[i].getName());
			assertEquals(ids[i], dnas[i].getName());
		}
	}
	
	@Test
	public void testGetHomologs() throws IOException, IdNotRecognizedException {
		EnsembleWSReader ensReader = new EnsembleWSReader();
		String[] homologs = ensReader.getHomologs("ENSG00000122877", Species.RATTUS_NORVEGICUS);
		for (String string : homologs) {
			assertEquals(string, "ENSRNOG00000000640");
		}
		System.out.println(Arrays.deepToString(homologs));
	}

}
