/**
 * 
 */
package app.ensembl;


public enum Species {
	HOMO_SAPIENS("Homo sapiens", "hsapiens_gene_ensembl", "human_ensembl_gene"),
	MUS_MUSCULUS("Mus musculus", "mmusculus_gene_ensembl", "mouse_ensembl_gene"),
	RATTUS_NORVEGICUS("Rattus norvegicus", "rnorvegicus_gene_ensembl", "rat_ensembl_gene");
	private String fullName;
	private String dataSet;
	private String attribute;

	private Species(String fullName, String dataSet, String attribute) {
		this.fullName = fullName;
		this.dataSet = dataSet;
		this.attribute = attribute;
	}
	
	public String getSpeciesName() {
		return this.fullName;
	}
	
	public Species getFromFullName(String fullName) {
		Species[] species = Species.values();
		for (Species s : species) {
			if (s.getSpeciesName().equals(fullName))
				return s;
		}
		return null;
	}
	
	public String getExactName() {
		return name();
	}
	
	public static boolean isGene(String stableId) {
		return stableId.startsWith("ENSG") || stableId.startsWith("ENSRNOG") || stableId.startsWith("ENSMUSG");
	}
	
	public static Species getForEnsembleId(String ensemblId) throws IdNotRecognizedException {
		if (ensemblId.startsWith("ENSG") || ensemblId.startsWith("ENST"))
			return Species.HOMO_SAPIENS;
		else if (ensemblId.startsWith("ENSRNOG") || ensemblId.startsWith("ENSRNOT"))
			return Species.RATTUS_NORVEGICUS;
		else if (ensemblId.startsWith("ENSMUSG") || ensemblId.startsWith("ENSMUST"))
			return Species.MUS_MUSCULUS;
		else
			throw new IdNotRecognizedException(ensemblId);
	}
	
	public String getDataSet() {
		return this.dataSet;
	}
	
	public String asAttribute() {
		return attribute;
	}
}